//This file is part of the LDeque v0.0 library written in -*- C++ -*-

//LDeque v0.0: a simple portable deque class
//Copyright (C) Luca Saiu, 1999

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      luca78@dada.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


//To do: make LDeque resizable


#include <iostream.h>
#include <malloc.h>
#include "../Lgen/Lgen.h"

#define DIM 64

#define OverflowException   3244
#define UnderflowException  7458

template <class T> class LDeque{
protected:
  T *Buffer;            // Circular buffer
  long PushPopIndex;    // Where an element can be pushed (top)
  long EnqueueIndex;    // Where an element can be enqueued (bottom)
  int NFull;            // Number of full elements
public:
  // Constructors and destructor
  LDeque();
  ~LDeque();

  // Modifying Methods
  void Push(const T&);           // Push on the top
  const T Pop();                 // Pop from the top
  void Enqueue(const T&);        // Enqueue to the bottom
  const T Slide();               // Slide from the bottom

  // Query methods
  bool IsEmpty();
  bool IsFull();
  int GetMaxDim();
  int GetDim();
};

template <class T> LDeque<T>::LDeque():PushPopIndex(1),
                                       EnqueueIndex(0),
                                       NFull(0){
  Buffer=new T[DIM];
}

template <class T> LDeque<T>::~LDeque(){
  delete Buffer;
}

template <class T> bool LDeque<T>::IsFull(){
  return NFull==GetMaxDim();
}

template <class T> bool LDeque<T>::IsEmpty(){
  return NFull==0;
}

template <class T> void LDeque<T>::Enqueue(const T &x){
  if(IsFull()){
    throw OverflowException;
    return;
  }
  NFull++;
  Buffer[EnqueueIndex]=x;
  EnqueueIndex=(EnqueueIndex+GetMaxDim()-1)%GetMaxDim();
}

template <class T> const T LDeque<T>::Slide(){
  if(IsEmpty()){
    throw UnderflowException;
    fatal("LDeque<T>::Pop(): Underflow");
    // A compiler warning is avoided just because of the return
    // at the end of this method.
  }
  NFull--;
  EnqueueIndex=(EnqueueIndex+1)%GetMaxDim();
  return Buffer[EnqueueIndex];
}

template <class T> void LDeque<T>::Push(const T &x){
  if(IsFull()){
    throw OverflowException;
    return;
  }
  NFull++;
  Buffer[PushPopIndex]=x;
  PushPopIndex=(PushPopIndex+1)%GetMaxDim();
}

template <class T> const T LDeque<T>::Pop(){
  if(IsEmpty()){
    throw UnderflowException;
    fatal("LDeque<T>::Pop(): Underflow");
    // A compiler warning is avoided just because of the return
    // at the end of this method.
  }
  NFull--;
  PushPopIndex=(PushPopIndex+GetMaxDim()-1)%GetMaxDim();
  return Buffer[PushPopIndex];
}

template <class T> int LDeque<T>::GetDim(){
  return NFull;
}

template <class T> int LDeque<T>::GetMaxDim(){
  return DIM;
}

