//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "LWidget.h"
#include "LAttributes.h"
#include "LAttributeTable.h"
#include <stdarg.h>

#include "sysdep/LSysDep.h"

//#define DEBUG

long WidgetCounter=0;

LWidget::LWidget():parent(0){
  InsertString(LAName,false,Lstring("Un-named LWidget #")
                                       +Lstring(WidgetCounter++));
  InsertPointer(LAUserDefinedData,false,0);
#ifdef DEBUG
  cerr<<"An object derived from LWidget has been created\n";
#endif//ifdef DEBUG
}

LWidget::~LWidget(){
#ifdef DEBUG
  cerr<<"The \'"<<GetString(LAName)<<
        "\' object, which is derived from LWidget has been destroyed\n";
#endif//ifdef DEBUG
}

void LWidget::Set(LName n,...){
  va_list l;

  va_start(l,n);         // n is unuseful here, I wrote it just to follow the
                         // ANSI C++ Standard.
  while(n!=LAEnd){       // repeat while there are arguments remaining
    switch(n%TYPEWIDTH){ // look at the attribute type
      case LTInt:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Set(): This int attribute does not exist");
#endif//#ifdef DEBUG
        SetInt(n,va_arg(l,int)); // Write in the table
        break;
      }//LTInt:
      case LTDouble:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Set(): This double attribute does not exist");
#endif//#ifdef DEBUG
        SetDouble(n,va_arg(l,double)); // Write in the table
        break;
      }//LTDouble:
      case LTBool:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Set(): This bool attribute does not exist");
#endif//#ifdef DEBUG
        SetBool(n,va_arg(l,int)); // Write in the table
        break;
      }//LTBool:
      case LTString:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Set(): This string attribute does not exist");
#endif//#ifdef DEBUG
        SetString(n,va_arg(l,char*)); // Write in the table
        break;
      }//LTString:
      case LTPointer:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Set(): This pointer attribute does not exist");
#endif//#ifdef DEBUG
        SetPointer(n,va_arg(l,void*)); // Write in the table
        break;
      }//LTPointer:
      case LTCallBack:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Set(): This callback attribute does not exist");
#endif//#ifdef DEBUG
        SetCallBack(n,va_arg(l,LCallBack)); // Write in the table
        break;
      }//LTCallBack:
      default:{
        fatal("LWidget::Set() was called with a unknown-type attribute");
      }
    }//switch
    if(n!=LAEnd)           //if this argument is not the last...
      n=va_arg(l,LName);   //...retrieve the next attribute name
  }//while
  va_end(l);
  Apply();                 //Commits changes
  return;
}

void LWidget::Get(LName n,...){
  va_list l;
  
  va_start(l,n);         // n is unuseful here, I wrote it just to follow the
                         // ANSI C++ Standard.

  while(n!=LAEnd){       // repeat while there are arguments remaining
    switch(n%TYPEWIDTH){ // look at the attribute type
      case LTInt:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Get(): This int attribute does not exist");
#endif//#ifdef DEBUG
        *(va_arg(l,int*))=GetInt(n); // Write from table
        break;
      }//LTInt:
      case LTDouble:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Get(): This double attribute does not exist");
#endif//#ifdef DEBUG
        *(va_arg(l,double*))=GetDouble(n); // Write from table
        break;
      }//LTDouble:
      case LTBool:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Get(): This bool attribute does not exist");
#endif//#ifdef DEBUG
        *(va_arg(l,bool*))=GetBool(n); // Write from table
        break;
      }//LTBool:
      case LTString:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Get(): This string attribute does not exist");
#endif//#ifdef DEBUG
        *(va_arg(l,Lstring*))=GetString(n); // Write from table
        break;
      }//LTString:
      case LTPointer:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Get(): This pointer attribute does not exist");
#endif//#ifdef DEBUG
        *(va_arg(l,void**))=GetPointer(n); // Write from table
        break;
      }//LTPointer:
      case LTCallBack:{
#ifdef DEBUG
        if(Exists(n)==false)
          fatal("LWidget::Get(): This callback attribute does not exist");
#endif//#ifdef DEBUG
        *(va_arg(l,LCallBack*))=GetCallBack(n); // Write from table
        break;
      }//LTCallBack:
      default:{
        fatal("LWidget::Get() was called with a unknown-type attribute");
      }
    }//switch
    if(n!=LAEnd)           // if this argument is not the last...
      n=va_arg(l,LName);   // ...retrieve the next attribute name
  }//while
  va_end(l);
  return;
}

LContainer& LWidget::GetParent(){
  return *parent;
}

// These methods simply calls the protected LAttributeTable::GetXXXX methods;
// any intelligent C++ compiler will remove calls and inline them.
int LWidget::GetInt(const LName n){
  return LAttributeTable::GetInt(n);
}

double LWidget::GetDouble(const LName n){
  return LAttributeTable::GetDouble(n);
}

bool LWidget::GetBool(const LName n){
  return LAttributeTable::GetBool(n);
}

Lstring LWidget::GetString(const LName n){
  return LAttributeTable::GetString(n);
}

void* LWidget::GetPointer(const LName n){
  return LAttributeTable::GetPointer(n);
}

LCallBack LWidget::GetCallBack(const LName n){
  return LAttributeTable::GetCallBack(n);
}



