#include "LDrawable.h"
#include "../../LDrawable.h"

extern "C"{
#include <X11/Xlib.h>
#include <X11/Xcms.h>

#include <math.h>
}
#include "LSysDep.h"

LDrawable::LDrawable(){
  // Initialize graphic context:
  GraphicContext=XCreateGC(GlobalSysDep.display,          // display
                 DefaultRootWindow(GlobalSysDep.display), // (parent) drawable
                 0,0);                                    // valuemask / value
}

LDrawable::~LDrawable(){
  // Free the graphic context:
  XFreeGC(GlobalSysDep.display,    // display
          GraphicContext);         // graphic context
}

void LDrawable::Clear(){
  // Set the fill style in the GC:
  // First define the meaningful fileds of an XGCValues structure:
  GlobalSysDep.TempXGCValues.plane_mask=AllPlanes;
  GlobalSysDep.TempXGCValues.fill_style=FillSolid;
  GlobalSysDep.TempXGCValues.background=XBlackPixel(GlobalSysDep.display,
                                                    GlobalSysDep.screen);
  GlobalSysDep.TempXGCValues.line_style=LineSolid;//it was LineDoubleDash;

  // Then modify the GC:
  XChangeGC(GlobalSysDep.display,               // display
            GraphicContext,                     // graphic context
            GCPlaneMask|
            GCFillStyle                         // valuemask
            |GCBackground|GCLineStyle,
            &(GlobalSysDep.TempXGCValues)       // values
           );

  // Draw a filled (with _foreground_ color) rectangle:
  XFillRectangle(GlobalSysDep.display,           // display
             drawable,                           // drawable
             GraphicContext,                     // graphic context
             0,0,                                // x / y
             w,h                                 // width / heigth
            );

}

void LDrawable::DrawPixel(int x,int y){
  // Draw a pixel on the drawable:
  XDrawPoint(GlobalSysDep.display,           // display
             drawable,                       // drawable
             GraphicContext,                 // graphic context
             x,y);                           // x and y coordinates
}

void LDrawable::DrawPixel(int x,int y,const LColor c){
  // Save the current color and set a new one:
  LColor oldcolor=GetColor();
  SetColor(c);

  // Draw the pixel:
  DrawPixel(x,y);

  // Reset the old color:
  SetColor(oldcolor);
}

void LDrawable::DrawLine(int x1,int y1,int x2,int y2){
  // Draw on the drawable:
  XDrawLine(GlobalSysDep.display,           // display
            drawable,                       // drawable
            GraphicContext,                 // graphic context
            x1,y1,x2,y2);                   // coordinates
}

void LDrawable::DrawLine(int x1,int y1,int x2,int y2,const LColor c){
  // Save the current color and set a new one:
  LColor oldcolor=GetColor();
  SetColor(c);

  // Draw the pixel:
  DrawLine(x1,y1,x2,y2);

  // Reset the old color:
  SetColor(oldcolor);
}

void LDrawable::DrawFullRectangle(int x1,int y1,
                                  int x2,int y2){
  // To do
}

void LDrawable::DrawFullRectangle(int x1,int y1,
                                  int x2,int y2,
                                  const LColor c){
  // To do
}

void LDrawable::DrawFullTriangle(int x1,int y1,
                                 int x2,int y2,
                                 int x3,int y3){
  // To do
}

void LDrawable::DrawFullTriangle(int x1,int y1,
                                 int x2,int y2,
                                 int x3,int y3,
                                 const LColor c){
  // To do
}

unsigned long LDrawable::TrueColorLColor2Pixel(const LColor c) const{
  // * Each component must be 'reduced' from the 16-bit representation as
  //   LColor component to n bit, where n is the size of the part of pixel
  //   reserved to the component. This can be done with a right shift.
  // * A left shift is needed to place each component in its place within
  //   the word.
  // * Components must be OR-ed together.
  return ((c.r>>GlobalSysDep.PlacesToRShiftRed)<<GlobalSysDep.PlacesToLShiftRed)|
         ((c.g>>GlobalSysDep.PlacesToRShiftGreen)<<GlobalSysDep.PlacesToLShiftGreen)|
         ((c.b>>GlobalSysDep.PlacesToRShiftBlue)<<GlobalSysDep.PlacesToLShiftBlue);
}

const LColor LDrawable::TrueColorPixel2LColor(unsigned long p) const{
  // * Use masks to separate components within the pixel.
  // * Shift components to the right, so that least meaningful bits are used.
  // * Shift components to the left, so that they are 'normalized' with
  //   respect to MaxLColorComponent.
  // * Compose components into a LColor object using its constructor.
  return LColor(((p&GlobalSysDep.visual->red_mask)>>GlobalSysDep.PlacesToLShiftRed)<<GlobalSysDep.PlacesToRShiftRed,
                ((p&GlobalSysDep.visual->green_mask)>>GlobalSysDep.PlacesToLShiftRed)<<GlobalSysDep.PlacesToRShiftRed,
                ((p&GlobalSysDep.visual->blue_mask)>>GlobalSysDep.PlacesToLShiftRed)<<GlobalSysDep.PlacesToRShiftRed);
}

void LDrawable::SetColor(const LColor c){
  // Set the color to be used in the GC:

  // Here I assume a TrueColor class visual:
  GlobalSysDep.TempXGCValues.foreground=TrueColorLColor2Pixel(c);

  // Modify the graphic context:
  XChangeGC(GlobalSysDep.display,          // display
            GraphicContext,                // graphic context
            GCForeground,                  // valuemask
            &(GlobalSysDep.TempXGCValues)  // values
           );

  // Update CurrentColor:
  CurrentColor=c;
}

const LColor LDrawable::GetColor() const{
  // Trivial access method:
  return CurrentColor;
}

void LDrawable::CopyAreaFrom(int x1dest,int y1dest,
                             LDrawable &source,int x1,int y1,int x2,int y2){
  // Just call the Xlib function to copy a drawable area, without
  // worrying about whether drawables exist:
  XCopyArea(GlobalSysDep.display,           // display
            source.drawable,                // source drawable
            drawable,                       // destination drawable
            GraphicContext,                 // graphic context
            x1,y1,                          // source x / y
            x2-x1,                          // width
            y2-y1,                          // height
            x1dest,y1dest                   // destination x / y
           );
}

