//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "../../LTakeshi.h"
#include "../../LInputBox.h"
#include "../../LAttributes.h"

extern "C"{
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Form.h>
}

static void LInputBoxUpdateCallBack(Widget w,XtPointer callData,
                               XtPointer clientData){
  // This callback updates the value of the LAText attribute at every
  // change in text. callData contains a pointer to the LTakeshi widget.

  // Get the current string and caret position in the Athena widget
  String s;
  int    cursorposition,max=((LInputBox*)callData)->GetInt(LAMax);

  XtVaGetValues(w,
                XtNstring,      	&s,
                0);
  XtVaGetValues(((LInputBox*)callData)->SysDep.widget,
                XtNinsertPosition,	&cursorposition,
                0);

  // If LAText is too long or a '\n' has been inserted...
  if(((strlen(s)>max)&&(max>0))||(s[cursorposition]=='\n')){
    LBeep();
    // Restore the old value of the displayed string
    XtVaSetValues(w,
                  XtNstring,            ((LInputBox*)callData)->
                                          GetString(LAText).ToCharStar(),
                  0);

    // Restore the previous value of the caret position, equal to the
    // recorded position minus 1
    XtVaSetValues(((LInputBox*)callData)->SysDep.widget,
                  XtNinsertPosition,	cursorposition-1,
                  0);

    // Purists must forgive me, but this is the most intuitive way to do it.
    // This is the reason why 'goto' exists in C++: sometimes it's useful
    goto CALLBACK;
  }

  // Now s points to the actual string which is displayed: update the
  // LTakeshi attribute without using LWidget::Set(): that would cause
  // another change in the Xt attributes
  ((LInputBox*)callData)->SetString(LAText,	s);

CALLBACK:
  // If there is a non-null LAOnModify callback procedure, then call it
  LCallBack cb=((LInputBox*)callData)->GetCallBack(LAOnChange);
  if(cb!=0)
    cb(*((LInputBox*)callData),LVoidEvent);

  return;
}

void LInputBox::Apply(){
  if(TestAndSet(LAW))
    XtVaSetValues(SysDep.widget,XtNwidth,GetInt(LAW),
                                0); // Apply width changes
  if(TestAndSet(LAH))
    XtVaSetValues(SysDep.widget,XtNheight,GetInt(LAH),
                                0); // Apply height changes
  if(TestAndSet(LAX))
    XtVaSetValues(SysDep.widget,
                  XtNhorizDistance,	GetInt(LAX),
                  0); // Apply x position changes
  if(TestAndSet(LAY))
    XtVaSetValues(SysDep.widget,
                  XtNvertDistance,	GetInt(LAY),
                  0); // Apply y position changes

  if(TestAndSet(LAMax)){ // Apply LAMax changes
    int max=GetInt(LAMax);
    Lstring s=GetString(LAText);

    // If LAText now is too long, touch (in UNIX jargon) it
    if((s.getdim()>max)&&(max>0))
      SetString(LAText,s); // So LAText will be cut, and the cursor moved
  } // Apply LAMax changes

  if(TestAndSet(LAText)){
    // If LAText is too long or contains a '\n' then cut it
    Lstring s=GetString(LAText);
    int max=GetInt(LAMax),dim=s.getdim();
    int i; // Loop counter

    for(i=0;i<dim;i++)
      if(s[i]=='\n'){
        s.setdim(i);
        break;
      }

    if((dim>max)&&(max>0)){ // LAMax==0 means 'no limit'
      // Cut LAText
      s.setdim(max);
      SetString(LAText,s);
      Reset(LAText); // To avoid another call to Apply()

      // Now set the LAText value
      XtVaSetValues(SysDep.TextSourceWidget,
                    XtNstring,    s.ToCharStar(),
                    0); // Apply text changes

      // Move cursor to max
      XtVaSetValues(SysDep.widget,
                    XtNinsertPosition,	max,
                    0);
    }//if
    else{
      // Now set the LAText value
      XtVaSetValues(SysDep.TextSourceWidget,
                    XtNstring,    s.ToCharStar(),
                    0); // Apply text changes

      // Move cursor to dim
      XtVaSetValues(SysDep.widget,
                    XtNinsertPosition,	dim,
                    0);
    }//else
  }// Apply LAText changes
}

void LInputBox::SystemDependantInit(){
  SysDep.widget=XtVaCreateManagedWidget("LInputBox",
                  asciiTextWidgetClass,parent->SysDep.area,
                  XtNborderWidth,   1,
                  XtNbottom,        XtChainTop,
                  XtNtop,           XtChainTop,
                  XtNleft,          XtChainLeft,
                  XtNright,         XtChainLeft,
                  XtNjustify,       XtJustifyLeft,
                  XtNeditType,      XawtextEdit, // Allow reading and writing
                  XtNresize,        XawtextResizeNever, // Don't resize when
                                                 // text overflows
                  XtNhorizDistance, 1,
                  XtNvertDistance,  1,
                  XtNresizable,     TRUE,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                  0);

  // Make SysDep.TextSourceWidget have the value of the Athena
  // TextSourceWidget widget
  XtVaGetValues(SysDep.widget,
                XtNtextSource,      &SysDep.TextSourceWidget,
                0);

  // Add the callback to update the value of LAText
  XtAddCallback(SysDep.TextSourceWidget,XtNcallback,LInputBoxUpdateCallBack,
                this);

  Apply();
}

void LInputBox::SystemDependantDone(){
}




