//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#ifndef LSYSDEP_H_
#define LSYSDEP_H_
// X11 version

extern "C"{
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
}

#include "../../../Lhash.h"

#include "../../LTypes.h"

class SystemDependantGlobalData{   // A SystemDependantGlobalData object
                                   // named GlobalSysDep is declared in
                                   // sysdep/LSysDep.cc
protected:
  // Methods:
  void LookForDepthAndVisual();
  void ComputeTrueColorParameters();

public:
  // Public fields:
  XtAppContext appContext;         // application context
  Display *display;                // display
  Screen *ScreenStruct;            // screen
  int screen;                      // screen number
  Depth *depth;                    // depth
  Visual *visual;                  // visual
  int PlacesToLShiftRed;
  int PlacesToRShiftRed;
  int PlacesToLShiftGreen;
  int PlacesToRShiftGreen;
  int PlacesToLShiftBlue;
  int PlacesToRShiftBlue;
  Colormap colormap;               // colormap
  Drawable drawable;               // default drawable for the screen
  XGCValues TempXGCValues;         // Used to modify GC's: see Xlib
                                   // documentation and XChangeGC()
  // Constructor:
  SystemDependantGlobalData();
};

// External declaration of the one and only SystemDependantGlobalData object:
extern SystemDependantGlobalData GlobalSysDep;

class LWidget; // this is useful for the next declaration.

// An Xt action procedure or an Xt callback takes a Widget as a name, but
// sometimes I need to map an Xt Widget (it's already a pointer) into a
// pointer to an LTakeshi LWidget object, which contains other data.
// This hash table is used to map Widget's into LWidget*.
// Every object which uses the hash table has to create an entry in its
// SystemDependantInit() method, and must remove it in its
// SystemDependantDone() method. In LTakeshi/Xt there's a one-to-one
// corrispondence between Widgets and LWidgets, so there are no ambiguity
// problems.

// The hash table is currently used by LCanvas. [To do: update the comment]

#define WID2WID_DIMENSION   256  // Dimension of hash table:
                                 // I want it to be rather fast.

// Hash table external declaration (the definition is in system/LSysDep.cc):
extern Lhash<LWidget*,Widget,WID2WID_DIMENSION> WidgetToLWidgetStar;

// Locate the LTakeshi LWidget from the Xt Widget w and call a LTakeshi
// CallBack function associated with it (useful in Xt action procedures):
void CallLCallBack(Widget w,LName CallBackName,const LEvent e);

// Converts from an X keycode as stored in an XKeyEvent structure to
// a LTakeshi LKey:
LKey keycode2LKey(unsigned int button);

#endif //#ifdef LSYSDEP_H_

