//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


extern "C"{
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>

#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/MenuButton.h>
}

#include "../../LWindow.h"
#include "../../LAttributes.h"

extern SystemDependantGlobalData SysDep; // This is defined in
                                               // sysdep/SysDepAsWindow.cc

void LWindow::Apply(){
  if(TestAndSet(LAX))
    XtVaSetValues(SysDepAsWindow.shell,XtNx,GetInt(LAX),
                               0); // Apply x position changes
  if(TestAndSet(LAY))
    XtVaSetValues(SysDepAsWindow.shell,XtNy,GetInt(LAY),
                               0); // Apply y position changes
  if(TestAndSet(LAW)){
    int w=GetInt(LAW);

    XtVaSetValues(SysDep.area,
                  XtNwidth,  w,
                  0);
    XtVaSetValues(SysDepAsWindow.statusbar,
                  XtNwidth,  w,
                  0);

    // Move SysDepAsWindow.stupid to avoid Athena messes up width:
    XtVaSetValues(SysDepAsWindow.stupid,
                  XtNhorizDistance, w-SysDepAsWindow.distance-1,
                  0);
  }// Apply width changes

  if(TestAndSet(LAH)){
    int h=GetInt(LAH);

    XtVaSetValues(SysDep.area,
                  XtNheight,   h,
                  0); // Apply height changes

    // Move SysDepAsWindow.stupid to avoid Athena messes up height:
    XtVaSetValues(SysDepAsWindow.stupid,
                  XtNvertDistance, h-SysDepAsWindow.distance-1,
                  0);
  } // Apply height changes

  if(TestAndSet(LATitle))
    XtVaSetValues(SysDepAsWindow.shell,
                  XtNtitle,	GetString(LATitle).ToCharStar(),
                  0); // Make the window manager set the window title

  if(TestAndSet(LAStatusBarText))
    XtVaSetValues(SysDepAsWindow.statusbar,
                  XtNlabel,	GetString(LAStatusBarText).ToCharStar(),
                  XtNwidth,	GetInt(LAW), // I _don't_ want the default
                                             // width!
                  0); // Apply statusbar text changes

  if(TestAndSet(LAModal)){ // Apply LAModal changes
    // If the status has changed and the window is already shown...
    if((SysDepAsWindow.IsModal!=GetBool(LAModal))&&(SysDepAsWindow.IsShown==true)){
      // Invert the value of SysDepAsWindow.IsModal, then add or remove an Xt
      // toolkit grab of the shell widget
      if(SysDepAsWindow.IsModal=(!SysDepAsWindow.IsModal)) // if now the window is modal...
        XtAddGrab(SysDepAsWindow.shell,            // ...then add the Xt toolkit grab
                  False,    // 'exclusive' flag
                  False);   // 'spring_loaded' flag
      else                                 // but if the window isn't modal...
        XtRemoveGrab(SysDepAsWindow.shell);        // remove it
    }
    else
      // The status has not changed ot the window is hidden: just update the
      // internal status
      SysDepAsWindow.IsModal=GetBool(LAModal);
  }                        // Apply LAModal changes: end
}

void LWindow::SystemDependantInit(){
  // Initlialize local data to store the current window status
  SysDepAsWindow.IsShown=SysDepAsWindow.IsModal=false;

  // Create a shell with a form child: in this way every window is not
  // related to the other ones, so that all windows are 'primary'. This
  // simplifies programming very much.

  // Create the Xt shell widget
  SysDepAsWindow.shell=XtVaAppCreateShell(0,"LShell",
                                  applicationShellWidgetClass,
                                  // I use an applicationShellWidgetClass to
                                  // allow the X window-manager to set the
                                  // title of the window
               GlobalSysDep.display,
               XtNallowShellResize,  TRUE,  // Clients should be able to
                                            // resize themselves
               XtNinput,             TRUE,  // Window can be focused
               XtNmappedWhenManaged, FALSE, // Show descendantes of the
                                            // shell only when it's
                                            // explicitally mapped
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
               0);

  // Create the Xt/Athena 'skeleton' widget
  SysDepAsWindow.widget=XtVaCreateManagedWidget("LWindowSkeleton",
                boxWidgetClass,	SysDepAsWindow.shell,
                // The menu bar is on the top, under it there's the proper
                // window, and under the window there's the status bar:
                // orientation is vertical
                XtNorientation,	XtorientVertical,

                XtNsensitive,	TRUE,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                0);

  // Create the Xt/Athena menubar widget
  SysDepAsWindow.menubar=XtVaCreateManagedWidget("LWindowMenuBar",
                 boxWidgetClass, SysDepAsWindow.widget,
                 // The bar border must be _invisible_: this menu bar is
                 // only glue to keep together menu buttons
                 XtNborderWidth,	0,
                 XtNhSpace,		3,
                 XtNvSpace,		0,
                 // Menu buttons are on an horizontal bar
                 XtNorientation,	XtorientHorizontal,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                 0);

  // Create the window as an Athena form widget
  SysDep.area=XtVaCreateManagedWidget("LWindow",
                formWidgetClass,     SysDepAsWindow.widget,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                0);

  // Find defaultDistance, to be used to place SysDepAsWindow.stupid:
  XtVaGetValues(SysDep.area,
              XtNdefaultDistance,    &(SysDepAsWindow.distance),
              0);

  // Create the stupid widget: see sysdep/LWindow.h for an explanation.
  // It will be placed in the right position when the LWindow LTakeshi
  // widget is sized.
  SysDepAsWindow.stupid=XtVaCreateManagedWidget("LStupid",
                        labelWidgetClass,  SysDep.area,
                        XtNlabel,            "",
                        XtNborderWidth,      0,
                        XtNheight,           1,
                        XtNwidth,            1,
                        XtNbottom,           XtChainTop,
                        XtNtop,              XtChainTop,
                        XtNleft,             XtChainLeft,
                        XtNright,            XtChainLeft,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                        0);

  // Create the statusbar as an Athena label widget
  SysDepAsWindow.statusbar=XtVaCreateManagedWidget("LWindowStatusBar",
                   labelWidgetClass, SysDepAsWindow.widget,
                   XtNjustify,       XtJustifyLeft,
                   XtNallowShellResize,  False,//TRUE,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                   0);

  Apply();
}

void LWindow::SystemDependantDone(){
  // First hide the window, to remove the toolkit grab if needed.
  Hide();

  // Destroy the Xt shell widget: this recursively destroys Xt children
  // widgets, so there is no need to destroy every non-window Xt child.
  XtDestroyWidget(SysDepAsWindow.shell);
}

void LWindow::Show(){
  // If the window is already shown then do nothing
  if(SysDepAsWindow.IsShown)
    return;

  // Realize and map (for the first time)
  XtRealizeWidget(SysDepAsWindow.shell);
  // Now the Xt form widgets have their default width and height, which are
  // computed at realization-time.

  // Set the right values for width and height
  XtVaSetValues(SysDep.area,
                XtNwidth,	GetInt(LAW),
                XtNheight,	GetInt(LAH),
                0);
  XtVaSetValues(SysDepAsWindow.statusbar,
                XtNwidth,	GetInt(LAW),
                0);

  // Show the Xt shell widget and its children
  XtMapWidget(SysDepAsWindow.shell);

  // If the window is modal then add an Xt toolkit grab
  if(SysDepAsWindow.IsModal)
    XtAddGrab(SysDepAsWindow.shell,// ...then add the Xt toolkit grab
              False,       // 'exclusive' flag
              False);      // 'spring_loaded' flag

  // Now the window is shown: set SysDepAsWindow.IsShown to true
  SysDepAsWindow.IsShown=true;
}

void LWindow::Hide(){
  // If the window is not shown then do nothing, and exit now
  if(SysDepAsWindow.IsShown==false)
    return;

  // if the window is modal then remove the Xt toolkit grab
  if(SysDepAsWindow.IsModal)
    XtRemoveGrab(SysDepAsWindow.shell);

  // Unmap the Xt shell widget
  XtUnmapWidget(SysDepAsWindow.shell);

  // Now the window isn't shown: set SysDepAsWindow.IsShown to false
  SysDepAsWindow.IsShown=false;
}






