#ifndef _LARRAY_H
#define _LARRAY_H

#include <stdio.h>
#include <iostream.h>
#include <malloc.h>
#include <stdlib.h>
#include "Lgen/Lgen.h"

template <class T> class Larray
{
public:
 Larray(INDEX=0);             //Just to declare size
 Larray(const Larray<T> &x);  //Copy constructor
 ~Larray();
 inline INDEX getdim() const; //Returns the number of the elements
 void setdim(INDEX);          //Changes the number of the elements
 T& operator[](INDEX);        //Instantiate an element
 int insert(INDEX);           //Adds an element at the n-th position
                              //right-shifting all v[x]:x>n
 int insert(INDEX,T);         //Adds an element at the n-th position
                              //right-shifting all v[x]:x>n
 int remove(INDEX);           //Destroys the n-th element, left-shifting
                              //all v[x]:x>n
 friend ostream& operator<< <>(ostream&,const Larray<T>&);
 const Larray<T>& operator=(const Larray<T> &x);
protected:
 INDEX d;
 T** p;
};

template <class T> INDEX Larray<T>::getdim() const{
  return d;
}

template <class T> int Larray<T>::insert(INDEX n)
{
 INDEX ct;

 if(n>getdim())
  return(-1);//Error: wrong n
 setdim(getdim()+1);
 for(ct=getdim()-1;ct>n;ct--)
  p[ct]=p[ct-1];
 p[n]=new(T);
 return(0);//OK
}

template <class T> int Larray<T>::insert(INDEX n, T t)
{
 int r=insert(n);
 if(!r)
  operator[](n)=t;
 return(r);
}

template <class T> int Larray<T>::remove(INDEX n)
{
 INDEX ct; 

 if((n>getdim())||(n<0))
  return(-1);//Error: wrong n
 else
 {
  delete(p[n]);
  for(ct=n;ct<getdim()-1;ct++)
   p[ct]=p[ct+1];
  d--;
  p=(T**)xrealloc(p,sizeof(T)*d);
  return(0);//OK
 }
}

template <class T> T& Larray<T>::operator[](INDEX n){
  if(n<d)
    return(*p[n]);
  else{
    fatal("Larray<T>::operator[](): out of bound");
    return *(p[0]);//just to avoid a compiler warning
  }
}

template <class T> void Larray<T>::setdim(INDEX n)
{
 INDEX ct; 
 
 if(d>n)
 {
  for(ct=n;ct<d;ct++)
   delete(p[ct]);
  p=(T**)xrealloc(p,(sizeof(T*))*n);//it was T**
 }
 else
 {
  p=(T**)xrealloc(p,(sizeof(T*))*n);//it was T**
  for(ct=d;ct<n;ct++)
   p[ct]=new(T);
 }
 d=n;
 return;
}

template <class T> Larray<T>::Larray(const Larray<T> &x){
  int i;

  p=NULL;
  d=0;
  setdim(0);
  setdim(x.getdim());
  for(i=0;i<getdim();i++)
    operator[](i)=*(x.p[i]);
  return;
}

template <class T> const Larray<T>& Larray<T>::operator=(const Larray<T> &x){
  int i;
  setdim(x.getdim());
  for(i=0;i<getdim();i++)
    operator[](i)=*(x.p[i]);
  return *this;
}

template <class T> Larray<T>::Larray(INDEX s)
{
 p=NULL;
 d=0;
 setdim(0);
 setdim(s);
}

template <class T> Larray<T>::~Larray()
{
 setdim(0);
}

template <class T> ostream& operator<<(ostream& s,const Larray<T>& b)
{
 Larray<T> A=b;//very dirty hack; To do: fix in a more efficient way;
               //I want the array parameter to be const

 INDEX ct;
 s<<"#(";
 for(ct=0;ct<A.getdim();ct++)
 {
  s<<A[ct];
  if(ct+1<A.getdim())
   s<<' ';
  else
   s<<')';
 }
 return(s);
}
#endif






