#ifndef _LTREE_H
#define _LTREE_H

#include "Lgen/Lgen.h"

template <class T> class Ltree{
 unsigned depth;//also an unsigned char would have been enough...
 int nsons;
 Ltree<T> *parent,**sons;
 T c;
public:
 Ltree();
 Ltree(Ltree<T>&);
 Ltree(T x);
 ~Ltree();
 inline T& access();
 inline Ltree<T> *accessparent();
 inline Ltree<T> *accessson(int);
 inline BOOL isleaf();
 inline unsigned getdepth();
 inline int getnsons();
 int createson(int,T);
 int removeson(int);
 friend ostream& operator<<(ostream&,Ltree<T>&);
};

template <class T> T& Ltree<T>::access(){
 return c;
}

template <class T> Ltree<T>::Ltree(T x){
 parent=0;
 depth=0;
 nsons=0;
 sons=0;
 c=x;
}

template <class T> Ltree<T>::Ltree(){
 parent=0;
 depth=0;
 nsons=0;
 sons=0;
}

template <class T> Ltree<T>::Ltree(Ltree<T> &T){
 c=T.c;
 depth=T.depth;
 int ct;
 parent=0;
 nsons=T.nsons;
 sons=(Ltree<T>**)xmalloc(nsons*sizeof(Ltree<T>*));
 for(ct=0;ct<nsons;ct++)
  if(T.accessson(ct)){
   sons[ct]->parent=this;
   sons[ct]=new Ltree(*T.sons[ct]);
  }
  else
   sons[ct]=0;
}

template <class T> Ltree<T>::~Ltree(){
 int ct;
 cerr<<"Destroying \'"<<c<<"\'..."<<endl;
 for(ct=0;ct<nsons;ct++)
  if(sons[ct])
   delete sons[ct];
 free(sons);
 cerr<<"...done"<<endl;

}

template <class T> int Ltree<T>::createson(int i,T x){
 int r=EXIT_FAILURE;
 if(i>nsons-1){
  sons=(Ltree<T>**)xrealloc(sons,(i+1)*sizeof(Ltree<T>*));
  int ct;
  for(ct=nsons;ct<=i;ct++)
   sons[ct]=0;
 }
 nsons=i+1;
 if(!sons[i]){
  sons[i]=new Ltree<T>;
  sons[i]->parent=this;
  sons[i]->depth=depth+1;
  r=EXIT_SUCCESS;
 }
 sons[i]->c=x;
 return r;
}

template <class T> int Ltree<T>::removeson(int i){
 if((i<0)||(i>=nsons))
  return EXIT_FAILURE;
 if(sons[i]){
  delete sons[i];
  sons[i]=0;
  int lastexistingson;
  for(lastexistingson=nsons-1;!sons[lastexistingson];lastexistingson--)
   sons=(Ltree<T>**)xrealloc(sons,sizeof(Ltree<T>*)*(lastexistingson+1));
  nsons=lastexistingson+1;
  return EXIT_SUCCESS;
 }
 else
  return EXIT_FAILURE;
}

template <class T> BOOL Ltree<T>::isleaf(){
 int ct;
 BOOL r=TRUE;
 for(ct=0;(ct<n)&&r;ct++)
  if(sons[ct])
   r=FALSE;
 return r;
}

template <class T> Ltree<T>* Ltree<T>::accessparent(){
 return parent;
}

template <class T> Ltree<T>* Ltree<T>::accessson(int i){
 if(sons[i])
  return sons[i];
 else
  return 0;
}

template <class T> unsigned Ltree<T>::getdepth(){
 return depth;
}

template <class T> int Ltree<T>::getnsons(){
 return nsons;
}

#endif

